unit xmlrpcclient;

interface
uses
  classes,
  sysutils,
  xmlrpctypes,
  xmlrpccommon,
  IdHTTP,
  {$IFDEF WIN32}
  dialogs;
  {$ENDIF}
  {$IFDEF LINUX}
   QDialogs;
  {$ENDIF}
type

    TSimpleParser = class(TObject)
    private
      FTagStack: TStrings;
      FVersionSent: boolean;
      function StripVersionTag(sData: string): string;
      function StripData(sData: string): string;
      function StripDataTag(sData: string): string;
      function StripTag(sData: string):string;
      function IsVersionTag(sData: string): boolean;
      function IsDataTag(sData: string):boolean;
      function IsStartTag(sData: string):boolean;
      function IsEndTag(sData: string):boolean;
      procedure FireDataEvent(sData: string);
    public
      constructor Create;
      procedure StartTag(sTagName: string); virtual;
      procedure EndTag(sTagName: string); virtual;
      procedure DataTag(sTagData: string); virtual;
      procedure VersionTag(sTagVersion: string);virtual;
      function LoadFromString(sXmlData: string): boolean;
      function LoadFromFile(sXmlFile: string):boolean;
      procedure Parse;
      destructor Destroy; override;
    end;

   TRPCParser = class(TSimpleParser)
  private
    FMode:TDataType;
    FNestMode: TDataType;
    FResult: TResult;
    FName: string;
  public
    constructor Create;
    procedure Clear;
    function GetResult: TResult;
    procedure StartTag(sTagName: string); override;
    procedure EndTag(sTagName: string); override;
    procedure DataTag(sTagData: string); override;
    procedure VersionTag(sTagVersion: string);override;
    destructor Destroy; override;
  end;

  TCaller = class(TRPCParser)
    private
      FHostName: string;
      FHostPort: integer;
      FProxyName: string;
      FProxyPort: integer;
      FProxyUserName: string;
      FProxyPassword: string;
      FSSLEnable: boolean;
      FEndPoint: string;
      function Post(rdata: string): string;
    public
      procedure Clear;
      property EndPoint: string read FEndPoint write FEndPoint;
      property HostName: string read FHostName write FHostName;
      property HostPort: integer read FHostPort write FHostPort;
      property ProxyName: string read FProxyName write FProxyName;
      property ProxyPort: integer read FProxyPort write FProxyPort;
      property ProxyUserName: string read FProxyUserName write FProxyUserName;
      property ProxyPassword: string read FProxyPassword write FProxyPassword;
      property SSLEnable: boolean read FSSLEnable write FSSLEnable;
      function Execute(value: TFunction): TResult;
    end;
implementation

{ TCaller }


{------------------------------------------------------------------------------}
{******************************************************************************}
{******************************************************************************}
{  TSIMPLEPARSER CLASS                                                           }
{******************************************************************************}
{******************************************************************************}
{------------------------------------------------------------------------------}

{------------------------------------------------------------------------------}
{  constructor                                                                 }
{------------------------------------------------------------------------------}
constructor TSimpleParser.Create;
begin
  FTagStack := TStringList.Create;
  FVersionSent := False;
end;

{------------------------------------------------------------------------------}
{  overridden data tag handler                                                 }
{------------------------------------------------------------------------------}
procedure TSimpleParser.DataTag(sTagData: string);
begin
end;

{------------------------------------------------------------------------------}
{  destructor                                                                  }
{------------------------------------------------------------------------------}
destructor TSimpleParser.Destroy;
begin
  inherited;
  FTagStack.Clear;
  FTagStack.Free;
  FTagStack := nil;
end;

{------------------------------------------------------------------------------}
{  overridden end tag handler                                                  }
{------------------------------------------------------------------------------}
procedure TSimpleParser.EndTag(sTagName: string);
begin
end;

{------------------------------------------------------------------------------}
{  check tag and fire appropriate handler                                      }
{------------------------------------------------------------------------------}
procedure TSimpleParser.FireDataEvent(sData: string);
begin
{ if we already sent the version tag skip
   the check  }
 if not FVersionSent then
   begin
     if IsVersionTag(sData) then
        begin
          VersionTag(StripVersionTag(sData));
          FVersionSent := true;
          exit;
        end;
    end;

  if IsDataTag(sData) then
    begin
      {send the data}
        DataTag(StripData(sData));
      {send the tag}
        EndTag(StripTag(StripDataTag(sData)));
        exit;
    end;
  if IsStartTag(sData) then
    begin
      StartTag(StripTag(sData));
      exit;
    end;
end;

{------------------------------------------------------------------------------}
{  check tag for data                                                          }
{------------------------------------------------------------------------------}
function TSimpleParser.IsDataTag(sData: string): boolean;
begin
  result := false;
  if (pos('<', trim(sData)) > 1) then
    result := true;
end;

{------------------------------------------------------------------------------}
{  check tag for end tag                                                       }
{------------------------------------------------------------------------------}
function TSimpleParser.IsEndTag(sData: string): boolean;
begin
  result := false;
  if (pos('/',sData) > 0) then
    result := true;
end;

{------------------------------------------------------------------------------}
{  check tag for start tag                                                     }
{------------------------------------------------------------------------------}
function TSimpleParser.IsStartTag(sData: string): boolean;
begin
  result := false;
  if (pos('<',trim(sData)) = 1) and (pos('/',trim(sData)) = 0)  then
    result := true;
end;

{------------------------------------------------------------------------------}
{  check tag for version tag                                                   }
{------------------------------------------------------------------------------}
function TSimpleParser.IsVersionTag(sData: string): boolean;
begin
  result := false;
  if (pos('?xml version="1.0"?',sData) > 1) or
     (pos('?xml version = "1.0"?',sData) > 1) or
     (pos('?xml version ="1.0"?',sData) > 1) or
     (pos('?xml version= "1.0"?',sData) > 1) then
     result := true;
end;

{------------------------------------------------------------------------------}
{  load parser from a file                                                     }
{------------------------------------------------------------------------------}
function TSimpleParser.LoadFromFile(sXmlFile: string): boolean;
var
  FStream: TFileStream;
begin
  FVersionSent := false;
  result := true;
  try
    {load the file / convert to string and load the stack}
    FTagStack.Clear; //first clear the stack
    FStream := TFileStream.Create(sXmlFile,fmOpenRead);
    LoadFromString(StreamToString(FStream));
    FStream.Free;
  except
    result := false;
  end;
end;

{------------------------------------------------------------------------------}
{  load parser from a string                                                   }
{------------------------------------------------------------------------------}
function TSimpleParser.LoadFromString(sXmlData: string): boolean;
begin
  result := true;
  FVersionSent := false;
  try
    FTagStack.Clear;
    {load up the tag / data stack }
    ParseString(sXmlData,'>',FTagStack,False,True);
  except
    result := false;
  end;
end;

{------------------------------------------------------------------------------}
{  place parser in run mode                                                    }
{------------------------------------------------------------------------------}
procedure TSimpleParser.Parse;
var
  Index: integer;
begin
  for Index := 0 to FTagStack.Count - 1 do
    begin
      FireDataEvent(FTagStack[Index]);
    end;
end;

{------------------------------------------------------------------------------}
{  overridden start tag handler                                                }
{------------------------------------------------------------------------------}
procedure TSimpleParser.StartTag(sTagName: string);
begin
end;

{------------------------------------------------------------------------------}
{  strip data from a tag                                                       }
{------------------------------------------------------------------------------}
function TSimpleParser.StripData(sData: string): string;
var
 sTemp: string;
begin
  SetLength(sTemp,pos('<',sData));
  sTemp := Copy(sData,1,pos('<',sData) -1);
  result := sTemp;
end;

{------------------------------------------------------------------------------}
{  strip tag from data                                                         }
{------------------------------------------------------------------------------}
function TSimpleParser.StripDataTag(sData: string): string;
begin
  result := Copy(sData,pos('<',sData),length(sData) - pos('<',sData) +1);
end;

{------------------------------------------------------------------------------}
{  strip misc chars from tag                                                   }
{------------------------------------------------------------------------------}
function TSimpleParser.StripTag(sData: string): string;
var
 sTemp: string;
begin
  sTemp := StringReplace(sData,'<','',[rfReplaceAll]);
  sTemp := StringReplace(sTemp,'>','',[rfReplaceAll]);
  sTemp := StringReplace(sTemp,'/','',[rfReplaceAll]);
  result := sTemp;
end;

{------------------------------------------------------------------------------}
{  strip misc chars from version tag                                           }
{------------------------------------------------------------------------------}
function TSimpleParser.StripVersionTag(sData: string): string;
var
 sTemp: string;
begin
  sTemp := StringReplace(sData,'<','',[rfReplaceAll]);
  sTemp := StringReplace(sTemp,'>','',[rfReplaceAll]);
  result := trim(StringReplace(sTemp,'?','',[rfReplaceAll]));
end;

{------------------------------------------------------------------------------}
{  overridden version tag handler                                              }
{------------------------------------------------------------------------------}
procedure TSimpleParser.VersionTag(sTagVersion: string);
begin
end;

{------------------------------------------------------------------------------}
{******************************************************************************}
{******************************************************************************}
{  TRPCPARSER CLASS                                                         }
{******************************************************************************}
{******************************************************************************}
{------------------------------------------------------------------------------}

constructor TRPCParser.Create;
begin
  inherited;
  FResult := TResult.Create;
  FMode := dtNone;
end;

{------------------------------------------------------------------------------}
{  clear the parser                                                            }
{------------------------------------------------------------------------------}
procedure TRPCParser.Clear;
begin
 if assigned(FResult) then
   FResult.Clear; 
end;

{------------------------------------------------------------------------------}
{  parser data handler                                                         }
{------------------------------------------------------------------------------}
procedure TRPCParser.DataTag(sTagData: string);
var
  ar: TArray;
  st: TStruct;
begin

 { name value store for next pass }
  if (FMode = dtName) then
    begin
      FName := sTagData;
      exit;
    end;

 
    { fault mode}
     if FResult.IsError then
       begin
         if (FMode = dtInteger) then
           FResult.AddErrorCode(StrToInt(sTagData));
         if (FMode = dtString) then
           FResult.AddErrorString(sTagData);
         exit;
       end;

 { not struct or array or error then we
   must have a singular value }
  if not FResult.IsArray then
    if not FResult.IsStruct then
      if not FResult.IsError then
        begin
          if FMode = dtInteger then
            FResult.AddItem(StrToInt(sTagData));
          if FMode = dtString then
            FResult.AddItem(sTagData);
          if FMode = dtDouble then
            FResult.AddItem(StrToFloat(sTagData));
          if FMode = dtBoolean then
            if (sTagData = '1') then
              FResult.AddItem(true)
            else
              FResult.AddItem(false);
           if (FMode = dtBase64) then
              FResult.AddItemBase64Data(sTagData);
           if (FMode = dtDate) then
              FResult.AddItemDate(ISOToDateTime(sTagData));
          exit;
        end;

    { if the mode is struct}
    if FResult.IsStruct then
      begin
          {nested array}
          if FNestMode = dtArray then
             begin
               if FResult.GetStruct.Count = - 1 then
                 begin
                   ar := TArray.Create;
                   FResult.GetStruct.AddItem(FName,ar);
                 end;
               if not FResult.GetStruct.IsArray(FResult.GetStruct.Count -1) then
                 begin
                   ar := TArray.Create;
                   FResult.GetStruct.AddItem(FName,ar);
                 end;

               if (FMode = dtInteger) then
                 FResult.GetStruct.GetArray(FResult.GetStruct.Count -1).AddItem(StrToInt(sTagData));
               if (FMode = dtString) then
                 FResult.GetStruct.GetArray(FResult.GetStruct.Count -1).AddItem(sTagData);
               if (FMode = dtDouble) then
                 FResult.GetStruct.GetArray(FResult.GetStruct.Count -1).AddItem(StrToFloat(sTagData));
               if (FMode = dtDate) then
                 FResult.GetStruct.GetArray(FResult.GetStruct.Count -1).AddItem(ISOToDateTime(sTagData));
               if (FMode = dtBase64) then
                 FResult.GetStruct.GetArray(FResult.GetStruct.Count -1).AddItemBase64Data(sTagData);
               if (FMode = dtBoolean) then
                 if (sTagData = '1') then
                   FResult.GetStruct.GetArray(FResult.GetStruct.Count -1).AddItem(true)
                 else
                   FResult.GetStruct.GetArray(FResult.GetStruct.Count -1).AddItem(false);
               exit;
             end;

        {nested struct}
           if FNestMode = dtStruct then
             begin
               if FResult.GetStruct.Count = -1 then
                 begin
                   st := TStruct.Create;
                   FResult.GetStruct.AddItem(FName,st);
                 end;
               if not FResult.GetStruct.IsStruct(FResult.GetStruct.Count -1) then
                 begin
                   st := TStruct.Create;
                   FResult.GetStruct.AddItem(FName,st);
                 end;

              if(FMode = dtString) then
                FResult.GetStruct.GetStruct(FResult.GetArray.Count -1).AddItem(FName,sTagData);
              if(FMode = dtInteger) then
                FResult.GetStruct.GetStruct(FResult.GetArray.Count -1).AddItem(FName,StrToInt(sTagData));
              if(FMode = dtDouble) then
                FResult.GetStruct.GetStruct(FResult.GetArray.Count -1).AddItem(FName,StrToFloat(sTagData));
              if(FMode = dtBoolean) then
                if (sTagData = '1') then
                  FResult.GetStruct.GetStruct(FResult.GetArray.Count -1).AddItem(FName,true)
                else
                  FResult.GetStruct.GetStruct(FResult.GetArray.Count -1).AddItem(FName,false);
              if(FMode = dtDate) then
                FResult.GetStruct.GetStruct(FResult.GetArray.Count -1).AddItemDate(FName,ISOToDateTime(sTagData));
              if(FMode = dtBase64) then
                FResult.GetStruct.GetStruct(FResult.GetArray.Count -1).AddItemBase64Data(FName,sTagData);
             exit;
            end;

        { not nested }
        if (FMode = dtInteger) then
          FResult.GetStruct.AddItem(FName,StrToInt(sTagData));
        if (FMode = dtString) then
          FResult.GetStruct.AddItem(FName,sTagData);
        if (FMode = dtDouble) then
          FResult.GetStruct.AddItem(FName,StrToFloat(sTagData));
        if (FMode = dtBoolean) then
          if (sTagData = '1') then
            FResult.GetStruct.AddItem(FName,true)
          else
            FResult.GetStruct.AddItem(FName,false);
        if (FMode = dtBase64) then
           FResult.GetStruct.AddItemBase64Data(FName,sTagData);
        if (FMode = dtDate) then
           FResult.GetStruct.AddItemDate(FName,ISOToDateTime(sTagData));
      end;





    { array mode }
    if FResult.IsArray then
      begin
        {nested array}
          if (FNestMode = dtArray) then
              begin
                if (FMode = dtString) then
                  FResult.GetArray.GetArray(FResult.GetArray.Count -1).AddItem(sTagData);
                if (FMode = dtInteger) then
                  FResult.GetArray.GetArray(FResult.GetArray.Count -1).AddItem(StrToInt(sTagData));
                if (FMode = dtDouble) then
                  FResult.GetArray.GetArray(FResult.GetArray.Count -1).AddItem(StrToFloat(sTagData));
                if (FMode = dtBoolean) then
                  if (sTagData = '1') then
                    FResult.GetArray.GetArray(FResult.GetArray.Count -1).AddItem(true)
                  else
                    FResult.GetArray.GetArray(FResult.GetArray.Count -1).AddItem(false);
                if (FMode = dtDate) then
                    FResult.GetArray.GetArray(FResult.GetArray.Count -1).AddItemDate(ISOToDateTime(sTagData));
                if (FMode = dtBase64) then
                    FResult.GetArray.GetArray(FResult.GetArray.Count -1).AddItemBase64Data(sTagData);
                exit;
              end;

        {nested struct}
          if (FNestMode = dtStruct) then
            begin
              if (FResult.GetArray.Count = -1) then
                begin
                  st := TStruct.Create;
                  FResult.GetArray.AddItem(st);
                end;
              if not FResult.GetArray.IsStruct(FResult.GetArray.Count -1) then
                begin
                  st := TStruct.Create;
                  FResult.GetArray.AddItem(st);
                end;
              if(FMode = dtString) then
                FResult.GetArray.GetStruct(FResult.GetArray.Count -1).AddItem(FName,sTagData);
              if(FMode = dtInteger) then
                FResult.GetArray.GetStruct(FResult.GetArray.Count -1).AddItem(FName,StrToInt(sTagData));
              if(FMode = dtDouble) then
                FResult.GetArray.GetStruct(FResult.GetArray.Count -1).AddItem(FName,StrToFloat(sTagData));
              if(FMode = dtBoolean) then
                if (sTagData = '1') then
                  FResult.GetArray.GetStruct(FResult.GetArray.Count -1).AddItem(FName,true)
                else
                  FResult.GetArray.GetStruct(FResult.GetArray.Count -1).AddItem(FName,false);
              if(FMode = dtDate) then
                FResult.GetArray.GetStruct(FResult.GetArray.Count -1).AddItemDate(FName,ISOToDateTime(sTagData));
              if(FMode = dtBase64) then
                FResult.GetArray.GetStruct(FResult.GetArray.Count -1).AddItemBase64Data(FName,sTagData);
            exit;
           end;



        {not nested}
        if (FMode = dtInteger) then
          FResult.GetArray.AddItem(StrToInt(sTagData));
        if (FMode = dtString) then
          FResult.GetArray.AddItem(sTagData);
        if (FMode = dtDouble) then
          FResult.GetArray.AddItem(StrToFloat(sTagData));
        if (FMode = dtDate) then
          FResult.GetArray.AddItemDate(ISOToDateTime(sTagData));
        if (FMode = dtBoolean) then
          if (sTagData = '1') then
            FResult.GetArray.AddItem(true)
          else
            FResult.GetArray.AddItem(false);
        if (FMode = dtBase64) then
          FResult.GetArray.AddItemBase64Data(sTagData);
      end;


end;

{------------------------------------------------------------------------------}
{  destroy the parser                                                          }
{------------------------------------------------------------------------------}
destructor TRPCParser.Destroy;
begin
end;

{------------------------------------------------------------------------------}
{  parser end tag handler                                                      }
{------------------------------------------------------------------------------}
procedure TRPCParser.EndTag(sTagName: string);
var
  sw: string;
begin
  sw := UpperCase(trim(sTagName));


  if (sw = 'STRUCT') then
    if FNestMode = dtStruct then
      FNestMode := dtNone
    else
      FMode := dtNone;

  if (sw = 'ARRAY') then
    if FNestMode = dtArray then
      FNestMode := dtNone
    else
      FMode := dtNone;
      
  if (sw = 'INT') then
    FMode := dtNone;
  if (sw = 'INT4') then
    FMode := dtNone;
  if (sw = 'STRING') then
    FMode := dtNone;
  if (sw = 'DOUBLE') then
    FMode := dtNone;
  if (sw = 'BOOLEAN') then
     FMode := dtNone;
  if (sw = 'DATETIME.ISO8601') then
     FMode := dtNone;
  if (sw = 'BASE64') then
     FMode := dtNone;
  if (sw = 'VALUE') then
     FMode := dtNone;
  if (sw = 'NAME') then
     FMode := dtNone;
end;

{------------------------------------------------------------------------------}
{  return a handler                                                            }
{------------------------------------------------------------------------------}
function TRPCParser.GetResult: TResult;
begin
  result := FResult;
end;

{------------------------------------------------------------------------------}
{  parser start tag handler                                                    }
{------------------------------------------------------------------------------}
procedure TRPCParser.StartTag(sTagName: string);
var
 sw:string;
 st: TStruct;
 ar: TArray;
begin
  sw := UpperCase(trim(sTagName));
  { struct }
  if (sw = 'STRUCT') and not FResult.IsError then
   begin
     { nested struct }
     if (FResult.IsStruct) then
       begin
         FNestMode := dtStruct;
         exit;
       end;
     if(FResult.IsArray) then
       begin
         FNestMode := dtStruct;
         st := TStruct.Create;
         FResult.GetArray.AddItem(st);
         exit;
       end;
    FMode := dtStruct;
    st := TStruct.Create;
    FResult.AddItem(st);
   end;


  {array}
  if (sw = 'ARRAY') then
    begin
      {nested array}
      if FResult.IsArray then
        begin
          FNestMode := dtArray;
          ar := TArray.Create;
          FResult.GetArray.AddItem(ar);
          exit;
        end;

      {no result set}
      FMode := dtArray;
      ar := TArray.Create;
      FResult.AddItem(ar);
    end;

  if (sw = 'FAULT') then
    begin
      FMode := dtError;
      FResult.SetError;
    end;
  if (sw = 'INT') then
    FMode := dtInteger;
  if (sw = 'I4') then
    FMode := dtInteger;
  if (sw = 'BOOLEAN') then
    FMode := dtBoolean;
  if (sw = 'STRING') then
    FMode := dtString;
  if (sw = 'DATETIME.ISO8601') then
    FMode := dtDate;
  if (sw = 'BASE64') then
    FMode := dtBase64;
  if (sw = 'NAME') then
    FMode := dtName;
  {default to string}
  if (sw = 'VALUE') then
    FMode := dtString;
end;

{------------------------------------------------------------------------------}
{  version tag handler                                                         }
{------------------------------------------------------------------------------}
procedure TRPCParser.VersionTag(sTagVersion: string);
begin
end;



procedure TCaller.Clear;
begin
 inherited;
end;

function TCaller.Execute(value: TFunction): TResult;
var
 rs: string;
begin
  rs := Post(value.GetAsXML);
  if (rs <> '') then
    begin
      LoadFromString(rs);
      Parse;
      value.Clear;
   //   clear;
      result := GetResult;
    end
  else
    begin
      result := TResult.Create;
      result.SetError;
      result.AddErrorCode(99);
      result.AddErrorString('Server returned a empty response');
    end;
end;

function TCaller.Post(rdata: string): string;
var
 rsp: TMemoryStream;
 snd: TMemoryStream;
 FSession : TIdHttp;
begin
 rsp := TMemoryStream.Create;
 snd := TMemoryStream.Create;
 StringToStream(rdata,snd); //convert to a stream
 FSession := TIdHTTP.Create(nil);

 { * proxy setup *}
  if (FProxyName <> '') then
    begin
      FSession.Request.ProxyServer := FProxyName;
      FSession.Request.ProxyPort := FProxyPort;
      FSession.Request.ProxyUsername := FProxyUserName;
      FSession.Request.ProxyPassword := FProxyPassword;
    end
  else
    begin
      FSession.Host := FHostName;
      FSession.Port := FHostPort;
    end;

  FSession.Request.Accept := '*/*';
  FSession.Request.ContentType := 'text/xml';
  FSession.Request.Connection := 'Keep-Alive';
  FSession.Request.ContentLength := length(rdata);
  FSession.Post('http://' + FHostName +  FEndPoint,snd,rsp);
  result := StreamToString(rsp);
  FSession.Free;
  rsp.Free;
  snd.Free;

end;

end.
