unit xmlrpccommon;

interface

uses
  Classes,
  sysutils,
  {$IFDEF WIN32}
  windows,
  dialogs;
  {$ENDIF}
  {$IFDEF LINUX}
   QForms,
   QDialogs;
  {$ENDIF}

type

   TRC4Data= record
    Key: array[0..255] of byte;         { current key }
    OrgKey: array[0..255] of byte;      { original key }
  end;

  TRC4 = class(TObject)
    private
      FData: TRC4Data;
      procedure RC4Init(var Data: TRC4Data; Key: pointer; Len: integer);
      procedure RC4Burn(var Data: TRC4Data);
      procedure RC4Crypt(var Data: TRC4Data; InData, OutData: pointer; Len: integer);
      procedure RC4Reset(var Data: TRC4Data);
    public
      constructor Create(EncryptionKey: string);
      procedure EncryptStream(instream,outstream: TMemoryStream);
      function EncryptString(value: string):string;
      procedure DecryptStream(instream,outstream: TMemoryStream);
      function DecryptString(value: string):string;
      procedure BurnKey;
   end;




 { xml-rpc data types }
  TRPCDataType = (rpNone,rpString,rpInteger,
                  rpBoolean,rpDouble,
                  rpDate,rpBase64,
                  rpStruct,rpArray,
                  rpName, rpError);

function EncodeEntities(const sData: string):string;
function DecodeEntities(const sData: string):string;

function Replace(const sData: string; const sFind:
                 string; const sReplace: string): string;

function InStr(sStart: integer; const sData: string; const
               sFind: string): integer;

function Mid(const sData: string; nStart: integer): string;

function DateTimeToISO(const ConvertDate: TDateTime): string;

function ISOToDateTime(const ISOStringDate: string): TDateTime;

function ParseString(const SearchString: String;
                     const Delimiter: Char;
                     const Substrings: TStrings;
                     const AllowEmptyStrings: Boolean = False;
                     const ClearBeforeParse: Boolean = False): Integer;

function ParseStream(const SearchStream: TStream;
                     const Delimiter: Char;
                     const Substrings: TStrings;
                     const AllowEmptyStrings: Boolean = False;
                     const ClearBeforeParse: Boolean = False): Integer;

function URLEncode(Value : String) : String;
function StreamToString(const Stream: TStream): String;
procedure StringToStream(const Text: String;
                         const Stream: TMemoryStream);
const

   ValidURLChars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789$-_@.&+-!''*"(),;/#?:';

implementation

Function URLEncode(Value : String) : String;
Var I : Integer;
Begin
   Result := '';
   For I := 1 To Length(Value) Do
      Begin
         If Pos(UpperCase(Value[I]), ValidURLChars) > 0 Then
            Result := Result + Value[I]
         Else
            Begin
               If Value[I] = ' ' Then
                  Result := Result + '+'
               Else
                  Begin
                     Result := Result + '%';
                     Result := Result + IntToHex(Byte(Value[I]), 2);
                  End;
            End;
      End;
End;

function EncodeEntities(const sData: string):string;
begin
 result := StringReplace(sData,'&','&amp;',[rfReplaceAll]);
 result := StringReplace(result,'<','&lt;',[rfReplaceAll]);
 result := StringReplace(result,'>','&gt;',[rfReplaceAll]);
 result := StringReplace(result,'"','&quot;',[rfReplaceAll]);
 result := StringReplace(result,#39,'&apos;',[rfReplaceAll]);
end;

function DecodeEntities(const sData: string):string;
begin
 result := StringReplace(sData,'&lt;','<',[rfReplaceAll]);
 result := StringReplace(result,'&gt;','>',[rfReplaceAll]);
 result := StringReplace(result,'&quot;','"',[rfReplaceAll]);
 result := StringReplace(result,'&apos;',#39,[rfReplaceAll]);
 result := StringReplace(result,'&amp;','&',[rfReplaceAll]);
end;
{------------------------------------------------------------------------------}
{ String Parsing Routine                                                       }
{------------------------------------------------------------------------------}
function ParseString(const SearchString: String;
                     const Delimiter: Char;
                     const Substrings: TStrings;
                     const AllowEmptyStrings: Boolean;
                     const ClearBeforeParse: Boolean): Integer;

var
  Index: Integer;
  PrevCount: Integer;
  TempStr: String;

begin
  if (ClearBeforeParse) then
    SubStrings.Clear;

  PrevCount := Substrings.Count;

  { ensure that the last substring is found }
  TempStr := SearchString + Delimiter;

  while (Length(TempStr) > 0) do
    begin
    Index := Pos(Delimiter, TempStr);
    if ((Index > 1) or AllowEmptyStrings) then
      Substrings.Add(Copy(TempStr, 1, Index - 1));
    Delete(TempStr, 1, Index);
    end;

  Result := Substrings.Count - PrevCount;
end;

{------------------------------------------------------------------------------}
{ stream parser                                                                }
{------------------------------------------------------------------------------}
function ParseStream(const SearchStream: TStream;
                     const Delimiter: Char;
                     const Substrings: TStrings;
                     const AllowEmptyStrings: Boolean;
                     const ClearBeforeParse: Boolean): Integer;

begin
  Result := ParseString(StreamToString(SearchStream), Delimiter, SubStrings,
                        AllowEmptyStrings, ClearBeforeParse);
end;

{------------------------------------------------------------------------------}
{ convert stream to a string                                                   }
{------------------------------------------------------------------------------}
function StreamToString(const Stream: TStream): String;
begin
  Result := '';
  Stream.Seek(0, soFromBeginning);
  SetLength(Result, Stream.Size);
  Stream.Read(Result[1], Stream.Size);
end;

{------------------------------------------------------------------------------}
{  Converts a string to a stream                                               }
{------------------------------------------------------------------------------}
procedure StringToStream(const Text: String;
                         const Stream: TMemoryStream);
begin
  Stream.SetSize(Length(Text));
  Stream.Seek(0, soFromBeginning);
  Stream.Write(Text[1], Length(Text));
  Stream.Seek(0, soFromBeginning);
end;

{------------------------------------------------------------------------------}
{  Converts a date time to iso 8601 format                                     }
{------------------------------------------------------------------------------}
function DateTimeToISO(const ConvertDate: TDateTime): string;
begin
  result := FormatDateTime('yyyymmdd"T"hh:mm:ss',ConvertDate);
end;

{------------------------------------------------------------------------------}
{  Converts a ISO 8601 data to TDateTime                                       }
{------------------------------------------------------------------------------}
function ISOToDateTime(const ISOStringDate: string): TDateTime;
var
  st: string;
  sdate: string;
begin
  sdate := ISOStringDate;
  st := sdate[5] + sdate[6] + '/' + sdate[7] + sdate[8] +
        '/' + sdate[3] + sdate[4] + ' ' + sdate[10] + sdate[11] +
        sdate[12] + sdate[13] + sdate[14] + sdate[15] + sdate[16] +
        sdate[17];
  result := StrToDateTime(st);
end;

{------------------------------------------------------------------------------}
{  Returns part of a string                                                    }
{------------------------------------------------------------------------------}
function Mid(const sData: string; nStart: integer): string;
begin
  Result := copy(sData, nStart, Length(sData) - (nStart - 1));
end;

{------------------------------------------------------------------------------}
{  Find position of string in sub string                                       }
{------------------------------------------------------------------------------}
function InStr(sStart: integer; const sData: string; const
               sFind: string): integer;
var
  c: integer;
label
  SkipFind;
begin
  c := sStart - 1;
  repeat
    if c > length(sData) then
  begin
    c := 0;
      goto SkipFind;
      end;
        inc(c);
      until copy(sData, c, length(sFind)) = sFind;
   SkipFind:
   Result := c;
 end;

{------------------------------------------------------------------------------}
{  replace item in string                                                      }
{------------------------------------------------------------------------------}
function Replace(const sData: string; const sFind:
                 string; const sReplace: string): string;
var
 c: integer;
 sTemp, sTemp2: string;
begin
  sTemp := sData;
  c := InStr(1, sTemp, sFind);
  while c <> 0 do
    begin
      sTemp2 := copy(sTemp, 1, c - 1) + sReplace + mid(sTemp, c + length(sFind));
      sTemp := sTemp2;
      c := InStr(c + length(sReplace), sTemp, sFind);
    end;
   result := sTemp;
end;
{------------------------------------------------------------------------------}
{Initialize the RC4 Engine                                                     }
{------------------------------------------------------------------------------}
procedure TRC4.RC4Init(var Data: TRC4Data; Key: pointer; Len: integer);
var
  xKey: array[0..255] of byte;
  i, j: integer;
  t: byte;
begin
  if (Len<= 0) or (Len> 256) then
    raise Exception.Create('RC4: Invalid key length');
  for i:= 0 to 255 do
  begin
    Data.Key[i]:= i;
    xKey[i]:= PByte(integer(Key)+(i mod Len))^;
  end;
  j:= 0;
  for i:= 0 to 255 do
  begin
    j:= (j+Data.Key[i]+xKey[i]) and $FF;
    t:= Data.Key[i];
    Data.Key[i]:= Data.Key[j];
    Data.Key[j]:= t;
  end;
  Move(Data.Key,Data.OrgKey,256);
end;

{------------------------------------------------------------------------------}
{Burn Key data from memory                                                     }
{------------------------------------------------------------------------------}
procedure TRC4.RC4Burn(var Data: TRC4Data);
begin
  FillChar(Data,Sizeof(Data),$FF);
end;

{------------------------------------------------------------------------------}
{Crypt and decrypt routine                                                     }
{------------------------------------------------------------------------------}
procedure TRC4.RC4Crypt(var Data: TRC4Data; InData, OutData: pointer; Len: integer);
var
  t, i, j: byte;
  k: integer;
begin
  i:= 0;
  j:= 0;
  for k:= 0 to Len-1 do
  begin
    i:= (i+1) and $FF;
    j:= (j+Data.Key[i]) and $FF;
    t:= Data.Key[i];
    Data.Key[i]:= Data.Key[j];
    Data.Key[j]:= t;
    t:= (Data.Key[i]+Data.Key[j]) and $FF;
    PByteArray(OutData)[k]:= PByteArray(InData)[k] xor Data.Key[t];
  end;
end;

{------------------------------------------------------------------------------}
{Reset the data keys                                                           }
{------------------------------------------------------------------------------}
procedure TRC4.RC4Reset(var Data: TRC4Data);
begin
  Move(Data.OrgKey,Data.Key,256);
end;

{------------------------------------------------------------------------------}
{Remove keys from memory                                                       }
{------------------------------------------------------------------------------}
procedure TRC4.BurnKey;
begin
  RC4Burn(FData);
end;

{------------------------------------------------------------------------------}
{Decrypt a memory stream                                                       }
{------------------------------------------------------------------------------}
procedure TRC4.DecryptStream(instream,outstream: TMemoryStream);
begin
   outstream.SetSize(instream.Size);
   RC4Crypt(FData,instream.Memory, outstream.Memory,instream.Size);
   RC4Reset(FData);
end;

{------------------------------------------------------------------------------}
{Secrypt a string value                                                        }
{------------------------------------------------------------------------------}
function TRC4.DecryptString(value: string): string;
var
os: string;
begin
 SetLength(os,length(value));
  RC4Crypt(FData,pByteArray(value),pByteArray(os),length(os));
 result := os;
end;

{------------------------------------------------------------------------------}
{Encrypt stream data                                                           }
{------------------------------------------------------------------------------}
procedure TRC4.EncryptStream(instream,outstream: TMemoryStream);
begin
   outstream.SetSize(instream.Size);
   RC4Crypt(FData,instream.Memory, outstream.Memory,instream.Size);
   RC4Reset(FData);
end;

{------------------------------------------------------------------------------}
{Encrypt a string value                                                        }
{------------------------------------------------------------------------------}
function TRC4.EncryptString(value: string): string;
var
os: string;
begin
 SetLength(os,length(value));
 RC4Crypt(FData,pByteArray(value),pByteArray(os),length(os));
 result := os;
 RC4Reset(FData);
end;

constructor TRC4.Create(EncryptionKey: string);
begin
  {initialize encryption engine}
  RC4Init(FData,pByteArray(EncryptionKey),length(EncryptionKey));
end;

end.
