unit XMLcontainers;
{version 0.1 alpha ;-) 15/08/2001
 An implementation of a double linked list and
 a hash table with the Separate Chaining  technique as a
 collision resolution scheme.

Copyright (c) 2001 Dean Zobec
 dezobec@tin.it
This file is released under an MIT style license, as detailed at opensource.org, and
as part of the Michael Johnson's quickRTTI library .
Please leave this comment and the Credits in the text.


 CREDITS:
 -the simple containers in this library were inspired when working for a demo
 of the QuickRTTI component written by Michael Johnson
 Michael Johnson / Big Attic House
 father@bigattichouse.com
 www.bigattichouse.com
 His fantastic QuickRTTI library made it possible to add XML persistence to the
 Containers. Thank you Michael for your wonderful work!!

 - Julian Bucknall - the implementation of the double linked list with
 the paging system was taken from a Julian's  article that appeard
  on The Delphi Magazine issue 42 february 1999 title:
 Algorithms Alfresco: Back On The Chain Gang

 - Gregory H.Deatz - for a beautiful article about Hash Lists that appeared
 on the Delphi Informant Magazine: "Making a Hash of It, Setting up an Abstract
 Hash Table" Sept.1999.
  }



interface
uses SysUtils,classes;

Const InvalidClassTypeMsg='The class is not of the expected type';
Const ItemsPerPage=5;

Type
EInvalidClassType=Class(Exception);


PDZItem=^TDZItem;
TDZItem=record
  FNext:PDZItem;
  FPrev:PDZItem;
  FVariantValue:Variant;
  FString:String;
  FObject:TObject
  end;



TDZCompareFunction=function(Lhs:PDZItem; Rhs:PDZItem):integer;

//Must return a longint value in the range 0..TableSize, usually via a mod operator;
THashFunction = function( const S:String;
                      const TableSize : Longint) : Longint;

{templates for the iterators over the containers items:
 TFunctionIterator when the Iterator has to be defined as a standalone procedure;
 TFunctionMethodIterator when the iterator needs to be defined as a
 function method of an object

 they must return True to let the iteration continue or False to cause
 the Iteration to stop (and return the item that caused the stopping).
 CallbackData  pointer is simply to enable the user to pass any kind
 of record structure to the iteration routine, thus eliminating
 the need for global variables}

TFunctionIterator   = function( const pItem    : pDZItem;
                             const CallBackData : Pointer ):boolean;

TFunctionMethodIterator = function( const pItem  : pDZItem;
                             const CallBackData : Pointer ):boolean of object;



{TDZContainer is an abstract class that provides the interface to all the
 containers defined in this unit.}
TXMLContainer=class(TPersistent)
  private
 { Holds the class allowed in the container }
    FAllowedClassType   : TClass;
 { if True contained objects are disposed }
    FOwnsObjects:boolean;

 { number of elements in container }
    FCount      : LongInt;

 { Checks to make sure Obj is of FAllowedClassType and otherwise
  raises and EDZInvalidClassType exception
  if you use the XML persistence of the containers, the objects in the
  container must be of TPersistent type and be carefull to initialize
  all of the subobjects in an overridden afterconstruction method because
  this library is made for generic objects and uses the TObject's constructor
  for the objects when loading from XML}
    procedure VerifyClassType(const Obj : TObject);


  protected
 { GetCount and GetEmpty may overriden in descendant containers that
   use an encapsulated container }
    function GetCount   : LongInt; virtual;
    function GetEmpty : Boolean; virtual;



  public
    constructor Create(const AllowedClassType : TClass);  virtual;

 { The destructor calls Clear before destroying the container }
    destructor Destroy; override;
 { number of elements in container, read only property}
    property Count : LongInt read GetCount;

    { True if container is emtpy, read only }
    property Empty : Boolean read GetEmpty;



    { clear all the items in the container. }
    procedure Clear; virtual; abstract;


    function ForEachCall(const FunctionIterator : TFunctionIterator;
                          const ForwardDirection:Boolean;
                          const Data : Pointer ):PDZItem; Overload; virtual; abstract;
    function ForEachCall(const FunctionMethodIterator : TFunctionMethodIterator;
                          const ForwardDirection:Boolean;
                          const Data   : Pointer ):PDZItem; Overload; virtual; abstract;

    function SaveToXML (sTagName:String):String; virtual;
    procedure LoadFromXML(sTagName,sXML:String); virtual;
    { If OwnsObjects is True (the default), calling Clear frees all the objects
     in the container in addition to emptying it, calling the destructor
     frees also all the objects in the container}
    {TODO 1 -oDean -cPromemoria: remember to add the destruction of the object in case of OwnsObjects in the subclasses
     when deleting single items, and when assigning a different value to same key}
    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
    property AllowedClassType:TClass read FAllowedClassType;

end;


//Simple double linked list with some extra optimization.
TXMLDoubleLinkList=class(TXMLContainer)
  private
    FCursor : PDZItem;
    FBOL   : PDZItem;    //points to the beginning of the list
    FEOL   : PDZItem;
    FCompareFunction:TDZCompareFunction;
    procedure SetCompareFunction(const Value: TDZCompareFunction);    //points to the end of list
  protected

  public
  constructor Create(const AllowedClassType : TClass); override;
  { The destructor calls Clear before destroying the container }
  destructor Destroy; override;
  procedure InsertBeforeCursor(const aVariantValue:Variant; const aString:String; aObject:TObject);
  procedure InsertAfterCursor(const aVariantValue:Variant; const aString:String; aObject:TObject);
  function Delete:TObject;
  function IsEOL:boolean;
  function IsBOL:boolean;
  procedure MoveToEOL;
  procedure MoveToBOL;
  function MoveNext: boolean;
  function MovePrevious:boolean;
  procedure Clear; override;
  function Content:TObject;
  function StringContent:string;
  function GetCursor:PDZItem;
  function VariantValueContent:Variant;
  function Search(aVariantValue:variant; aString:string; aObject:TObject) : boolean;
  function ForEachCall(const FunctionIterator : TFunctionIterator;
                        const ForwardDirection:Boolean;
                        const Data : Pointer ):PDZItem; override;
  function ForEachCall(const FunctionMethodIterator : TFunctionMethodIterator;
                        const ForwardDirection:Boolean;
                        const Data   : Pointer ):PDZItem; override;
  function SaveToXML (sTagName:String):String; override;
  procedure LoadFromXML(sTagName,sXML:String); override;
  property CompareFunction:TDZCompareFunction read FCompareFunction write SetCompareFunction;

end;




THashItem=record
  List:TXMLDoubleLinkList;
  end;
PHashTableArray=^THashTableArray;
THashTableArray=array[0..0] of THashItem;


TXMLHashStringList=class(TXMLContainer)
private
    FHashTable:PHashTableArray;
    FHashTableSize:Longint;
    FHashFunction:THashFunction;
    FDuplicates:TDuplicates;
    FUpdateCount: Integer;
    FOnChange:TNotifyEvent;
    FOnChanging:TNotifyEvent;

    {uses ForEach to return all the strings in the container: obviously no particular
    order is assumed: as we say in Italy: non si pu avere la botte piena e la moglie ubriaca...}
    function GetTextString: string;

    {gets the value (right part after the =) associated with the Name string}
    function GetValue(const Name: string): string;
    procedure SetValue(const Name, Value: string);
    function GetVariantValue(const Name: string): Variant;
    procedure SetVariantValue(const Name:string; Value: Variant);
    function GetObject(S: String): TObject;
    procedure SetObject(S: String; const Obj: TObject);
    procedure SetTextString(const S: string);
    function GetShowStatistics: String;

protected
    procedure Changed; virtual;
    procedure Changing; virtual;
    procedure SetUpdateState(Updating: Boolean); virtual;
    function Find(const S:string; const RaiseException:boolean):PDZItem; virtual;

public
    constructor Create(const AllowedClassType: TClass; aHashTableSize:Longint;
                aHashFunction:THashFunction); reintroduce; Overload;
  { The destructor calls Clear before destroying the container }
    destructor Destroy; override;
    procedure Clear; override;
    procedure BeginUpdate;
    procedure EndUpdate;
    function Add(const S:string;const VariantValue:Variant):PDZItem;
    function AddObject(const S:string; const VariantValue:Variant; AObject:TObject):PDZItem;
    procedure AddStrings(Strings:TStrings);
    function Delete(const S:string):TObject;
    function ForEachCall(const FunctionIterator : TFunctionIterator;
                        const ForwardDirection:Boolean;
                        const Data : Pointer ):PDZItem; override;

    function ForEachCall(const FunctionMethodIterator : TFunctionMethodIterator;
                        const ForwardDirection:Boolean;
                        const Data   : Pointer ):PDZItem; override;

    function SaveToXML (sTagName:String):String; override;
    procedure LoadFromXML(sTagName,sXML:String); override;

    property Duplicates: TDuplicates read FDuplicates write FDuplicates default dupAccept;
    property Objects[S:String]:TObject read GetObject write SetObject;
    property Text: string read GetTextString write SetTextString;
    property Values[const Name: string]: string read GetValue write SetValue;
    property VariantValues[Const Name:string]:Variant read GetVariantValue write SetVariantValue;
    property ShowStatistics:String read GetShowStatistics;
    property OnChange:TNotifyEvent read FOnChange write FOnChange;
    property OnChanging:TNotifyEvent read FOnChanging write FOnChanging;
    property HashTableSize:Longint read FHashTableSize;
    property HashTable: PHashTableArray read FHashTable;
    end;

var
 {statistical counter of the number of pages allocated}
   NumPages:longint;
   NumItemsInRepository:longint;

function ShowPageStatistics:string;

{----------------A set of hash functions------------------------------------}
{I've added only 2 functions in the library by now, your function can as
 well be defined in a separate unit, the only need is to pass it to the
 constructor of the list}
function GetHashCode(Const S:String; Const TableSize: integer):Longint;
function SimpleHashFunction( const S:String;
                             const TableSize : Longint) : Longint;
{but you can of course use any hash function that is best suited for the
 data in your container. To do some tests of the hash function use the
 THashStringList.ShowStatistics function: the more chains you'll have and
 the shorter the average chain length the better is the performance
 of the hash function}
{---------------------------------------------------------------------------}

implementation
uses MiddleXRTTI,MiddleX;

{not a very good hash function, but simple to understand}
function SimpleHashFunction( const S:String;
                      const TableSize : Longint) : Longint;
var
  st: String;
  i: Integer;
begin
  st := S;
  result := 0;
  for i := 1 to Length(st) do Inc(result, Integer(st[i]));
  result := result mod TableSize;
end;

{taken with some modifications from DBTables.pas, a very good hash function
thank you Borland!}
function GetHashCode(Const S:String; Const TableSize:longint): Integer;
var
  Off, Len, Skip, I: Integer;

begin
  Result := 0;
  Off := 1;
  Len := Length(S);
  if Len < 16 then
    for I := (Len - 1) downto 0 do
    begin
      Result := (Result * 37) + Ord(S[Off]);
      Inc(Off);
    end
  else
  begin
    { Only sample some characters }
    Skip := Len div 8;
    I := Len - 1;
    while I >= 0 do
    begin
      Result := (Result * 39) + Ord(S[Off]);
      Dec(I, Skip);
      Inc(Off, Skip);
    end;
  end;
  Result:=Abs(Result) mod Tablesize;
end;

{the items of the double linked list are allocated in pages and stored in
a repository global to the  unit, when a new item is needed and there is no
item available in the  repository, a new page is allocated. Here comes the
implementation of the paging of the items}
type
  PPageOfItems=^TPageOfItems;
  TPageOfItems=record
    Items: array[0..ItemsPerPage-1] of TDZItem;
    NextPage:PPageOfItems;
    end;
var
   {global variable holding the pointer to the list of available items}
   pItemsRepository:PDZItem;
   {global variable holding the pointer to the list of allocated pages}
   pListOfPages:PPageOfItems;


procedure PlaceInRepository(pItem:PDZItem);
begin
 pItem^.FVariantValue:=Null;
 pItem^.FString:='';
 pItem^.FObject:=nil;
{insert the item as the first element in the repository}
pItem^.FNext:=pItemsRepository;
pItemsRepository:=pItem;
Inc(NumItemsInRepository);
end;

procedure AllocateNewPage;
var i:integer;
    NewPage:PPageOfItems;

begin
{allocate memory for the page}
New(NewPage);
{insert the new page in the list of allocated pages}
NewPage^.NextPage:=pListOfPages;
pListOfPages:=NewPage;
{insert the items contained in the page in the repository for future use}
for i:=0 to ItemsPerPage-1 do PlaceInRepository(@NewPage^.Items[i]);

Inc(NumPages);
end;

function AllocateNewItem:PDZItem;
begin
if pItemsRepository=nil then     //no available item in repository
  begin
  AllocateNewPage;
  end;
{take an Item out of the repository}
Result:=pItemsRepository;
pItemsRepository:=Result.FNext;
dec(NumItemsInRepository);
end;

function ShowPageStatistics:string;
begin
Result:='Number of pages: '+intToStr(numPages)+#10#13+
        'Number of items per page: '+intToStr(itemsPerPage)+#10#13+
        'Number of items available in repository: '+IntToStr(NumItemsInRepository);
end;

{extracts the name part - on the left of '=' from a string: only the name
    part of the string will be used to construct the hash key, the value part
    is only used to hold a string value, like in a dictionary...or in an
    .ini file}
function GetNamePart(S:string):string;
var p:Integer;
begin
Result:=S;
P := AnsiPos('=', S);
if P <> 0 then
    SetLength(Result, P-1);
end;

function GetValuePart(S:string):string;
var p:Integer;
begin
Result:='';
P := AnsiPos('=', S);
if P <> 0 then
    Result:=Copy(S, p+1, Maxint);
end;

function SetValuePart(S: string; Value:String):String;
var p:Integer;
begin
Result:=S;
P := AnsiPos('=', Result);
if P <> 0 then
    begin
    SetLength(Result, P);
    Result:=Result+Value;
    end;
end;


function compareByName(Lhs:PDZItem;Rhs:PDZItem):Integer;
begin
Result:=AnsiCompareText(GetNamePart(Lhs^.FString),GetNamePart(Rhs^.Fstring));     //upcase sensitive
end;

{ TXMLContainer }

constructor TXMLContainer.Create(const AllowedClassType: TClass);
begin
  FAllowedClassType  := AllowedClassType;
  FCount      :=0;
  FOwnsObjects:=true;
end;

destructor TXMLContainer.Destroy;
begin
  clear;
  inherited destroy;
end;

function TXMLContainer.GetCount: LongInt;
begin
Result:=FCount;
end;

function TXMLContainer.GetEmpty: Boolean;
begin
Result:=FCount=0;
end;


procedure TXMLContainer.LoadFromXML(sTagName, sXML: String);
begin

end;

function TXMLContainer.SaveToXML(sTagName: String): String;
begin

end;

procedure TXMLContainer.VerifyClassType(const Obj: TObject);
begin
  
  if Obj<>nil then
     begin
     if not (Obj is FAllowedClassType) then
        raise EInvalidClassType.create(InvalidClassTypeMsg);
     end;
end;

{ TXMLDoubleLinkList }

{Clears all the items in the list and places them in the repository for future
 reuse (or for reuse by another list) by default it also frees all the objects
 contained in the items, unless OwnsObjects is set to false;  }
procedure TXMLDoubleLinkList.Clear;
var temp:PDZItem;
begin
MoveToBOL;
if not empty then
  begin
  temp:=FBOL^.FNext;
  while (temp<>FEOL) do
    begin
    FBOL^.FNext:=temp^.FNext;
    if OwnsObjects then
      if (temp^.FObject<>nil) then freeAndNil(temp^.FObject);
    PlaceInRepository(temp);
    temp:=FBOL^.FNext;
    end;
  FCount:=0;
  FBOL^.FNext:=FEOL;
  FEOL^.FPrev:=FBOL;
  end;
end;

function TXMLDoubleLinkList.Content: TObject;
begin
Result:=FCursor^.FObject;
end;

function TXMLDoubleLinkList.StringContent: string;
begin
Result:=FCursor^.FString;
end;

function TXMLDoubleLinkList.VariantValueContent: Variant;
begin
Result:=FCursor^.FVariantValue;
end;

constructor TXMLDoubleLinkList.Create(const AllowedClassType: TClass);
begin
  inherited Create(AllowedClassType);
  FBOL:=AllocateNewItem;
  FEOL:=AllocateNewItem;
  FBOL^.FNext:=FEOL;
  FBOL^.FPrev:=nil;
  FBOL^.FObject:=nil;
  FBOL^.FString:='';
  FBOL^.FVariantValue:=Null;
  FEOL^.FNext:=nil;
  FEOL^.FPrev:=FBOL;
  FEOL^.FObject:=nil;
  FEOL^.FString:='';
  FEOL^.FVariantValue:=Null;
  FCursor:=FBOL; //position the cursor at the beginning of the list

end;

{also frees the object contained in the item, unless OwnsObject is false. In
this case it returns the object of the item that gets placed back in the repository}
function TXMLDoubleLinkList.Delete: TObject;
var ItemToBeDeleted:PDZItem;
begin
 ItemToBeDeleted:=FCursor;
 if (ItemToBeDeleted=FBOL) or (ItemToBeDeleted=FEOL) then
    raise Exception.create('Delete failed: cursor at the beginning or end of the list');
 if OwnsObjects then
   begin
   if (ItemToBeDeleted^.FObject<>nil) then FreeAndNil(ItemToBeDeleted^.FObject);
   result:=nil;
   end
 else Result:=ItemToBeDeleted^.FObject;

 ItemToBeDeleted^.FPrev^.FNext:=ItemToBeDeleted^.FNext;
 ItemToBeDeleted^.FNext^.FPrev:=ItemToBeDeleted^.FPrev;
 MoveNext; //moves the cursor to the next item in the list
 PlaceInRepository(ItemToBeDeleted);
 dec(FCount);
end;

destructor TXMLDoubleLinkList.Destroy;
begin
  Clear;  //also destroys all the contained objects unless OwnsObjects is set to false
  PlaceInRepository(FBOL);
  PlaceInRepository(FEOL);
  inherited destroy;
end;



function TXMLDoubleLinkList.ForEachCall(
            const FunctionMethodIterator: TFunctionMethodIterator;
            const ForwardDirection:Boolean;
            const Data: Pointer):PDZItem;
var Stop:boolean;
    FIteratorCursor:PDZItem;
begin
Stop:=false;
  if ForwardDirection then
    begin
    FIteratorCursor:=FBOL^.FNext;
    while ((FIteratorCursor<>FEOL) and (not Stop)) do
      begin
      Stop:=FunctionMethodIterator(FIteratorCursor,Data);
      if (not Stop) then FIteratorCursor:=FIteratorCursor^.FNext;;
      end;
    end
  else begin
      FIteratorCursor:=FEOL^.FPrev;
      while ((FIteratorCursor<>FBOL) and (not Stop)) do
        begin
        Stop:=FunctionMethodIterator(FIteratorCursor,Data);
        if (not Stop) then FIteratorCursor:=FIteratorCursor^.FPrev;
        end;
      end;

if (Not Stop) then
  begin
  Result:=nil;
  end
else begin
    Result:=FIteratorCursor;
    end;

end;




function TXMLDoubleLinkList.ForEachCall(
            const FunctionIterator: TFunctionIterator;
            const ForwardDirection:Boolean;
            const Data: Pointer):PDZItem;
var Stop:boolean;
    FIteratorCursor:PDZItem;
begin
Stop:=false;
  if ForwardDirection then
    begin
    FIteratorCursor:=FBOL^.Fnext;
    while ((FIteratorCursor<>FEOL) and (not Stop)) do
      begin
      Stop:=FunctionIterator(FIteratorCursor,Data);
      if (not Stop) then FIteratorCursor:=FIteratorCursor^.FNext;;
      end;
    end
  else begin
      FIteratorCursor:=FEOL^.FPrev;
      while ((FIteratorCursor<>FBOL) and (not Stop)) do
        begin
        Stop:=FunctionIterator(FIteratorCursor,Data);
        if (not Stop) then FIteratorCursor:=FIteratorCursor^.FPrev;
        end;
      end;

if (Not Stop) then
  begin
  Result:=nil;
  end
else begin
    Result:=FIteratorCursor;
    end;

end;



procedure TXMLDoubleLinkList.SetCompareFunction(
  const Value: TDZCompareFunction);
begin
 if Assigned(Value) then
    begin
    FCompareFunction := Value;
    //no sorting yet provided for the list otherwise would require
    //doing a sort if the list was not empty
    end;
end;

procedure TXMLDoubleLinkList.InsertAfterCursor(const aVariantValue:Variant; const aString:String; aObject:TObject);
var newItem:PDZItem;
begin
 VerifyClassType(aObject);
 if FCursor=FEOL then FCursor:=FCursor^.FPrev;
 newItem:=AllocateNewItem;
 newItem^.FVariantValue:=aVariantValue;
 newItem^.FString:=aString;
 newItem^.FObject:=aObject;
 newItem^.FNext:=FCursor^.FNext;
 newItem^.FPrev:=FCursor;
 FCursor^.FNext:=newItem;
 newItem^.FNext^.FPrev:=newItem;
 inc(FCount);
end;

procedure TXMLDoubleLinkList.InsertBeforeCursor(const aVariantValue:Variant; const aString:String; aObject:TObject);
var newItem:PDZItem;
begin
 VerifyClassType(aObject);
 if FCursor=FBOL then FCursor:=FCursor^.FNext;
 newItem:=AllocateNewItem;
 newItem^.FVariantValue:=aVariantValue;
 newItem^.FString:=aString;
 newItem^.FObject:=aObject;
 newItem^.FNext:=FCursor;
 newItem^.FPrev:=FCursor^.FPrev;
 newItem^.FPrev^.FNext:=newItem;
 FCursor^.FPrev:=newItem;
 inc(FCount);
end;



function TXMLDoubleLinkList.IsBOL: boolean;
begin
Result:=FCursor=FBOL;
end;

function TXMLDoubleLinkList.IsEOL: boolean;
begin
Result:=FCursor=FEOL;
end;

function TXMLDoubleLinkList.MoveNext: boolean;
begin
if FCursor=FEOL then Result:=false
  else begin
       FCursor:=FCursor^.FNext;
       Result:=true;
       end;
end;

function TXMLDoubleLinkList.MovePrevious: boolean;
begin
if FCursor=FBOL then Result:=false
  else begin
       FCursor:=FCursor^.FPrev;
       Result:=true;
       end;
end;

procedure TXMLDoubleLinkList.MoveToBOL;
begin
FCursor:=FBOL;
end;

procedure TXMLDoubleLinkList.MoveToEOL;
begin
FCursor:=FEOL;
end;

function TXMLDoubleLinkList.Search(aVariantValue:variant; aString:string; aObject:TObject): boolean;
var continue,found:boolean;
    SearchData:TDZItem;
    compareResult:Integer;
begin
moveToBOL;
SearchData.FVariantValue:=aVariantValue;
SearchData.FString:=aString;
SearchData.FObject:=aObject;
//linear search
continue := true;
found := false;
while continue and (not found) do
 begin

 FCursor := FCursor^.FNext;
 if FCursor=nil then
        continue := false
      else
        Begin
        CompareResult:=CompareFunction(@SearchData, FCursor);
        Found:=CompareResult=0;
        end;
 end;
if found then  FCursor := FCursor  //move the cursor to the found item
else MoveToBOL;
Result := found;
end;

function TXMLDoubleLinkList.GetCursor: PDZItem;
begin
Result:=FCursor;
end;

Procedure finalizeDZContainers;
var tempPage:PPageOfItems;
begin
 tempPage:=pListOfPages;
 while tempPage<>nil do
   begin
   pListOfPages:=tempPage^.NextPage;
   Dispose(tempPage);
   tempPage:=pListOfPages;
   end;
end;







{ TXMLHashStringList }

function TXMLHashStringList.Add(const S: String; const VariantValue:Variant): PDZItem;
var strKey:string;
    i:Longint;
    List:TXMLDoubleLinkList;
begin
Changing;
Result:=nil;
if not (Duplicates=dupAccept) then
   begin
   if Find(S, false)<>nil then
      case Duplicates of
        dupIgnore: Exit;
        dupError: raise Exception.Create('Duplicate string');
      end;
   end;

{extract the key from the S string in case its in the form 'Name=Value'); }
strKey:=GetNamePart(S);
i:=FHashFunction(strKey,FHashTableSize);
List:=FHashTable[i].List;
List.InsertAfterCursor(VariantValue,S,nil);
Inc(FCount);
List.MoveNext;
Result:=List.GetCursor;
Changed;
end;

function TXMLHashStringList.AddObject(const S: string; const VariantValue:Variant;
  AObject: TObject): PDZItem;
var strKey:string;
    i:Longint;
    List:TXMLDoubleLinkList;
begin
Changing;
Result:=nil;

if not (Duplicates=dupAccept) then
   begin
   if Find(S, false)<>nil then
      case Duplicates of
        dupIgnore: Exit;
        dupError: raise Exception.Create('Duplicate string');
      end;
   end;


{extract the key from the S string in case its in the form 'Name=Value'); }
strKey:=GetNamePart(S);
i:=FHashFunction(strKey,FHashTableSize);
List:=FHashTable[i].List;
List.InsertAfterCursor(VariantValue,S,AObject);
inc(FCount);
List.MoveNext;
Result:=List.GetCursor;
Changed;
end;

procedure TXMLHashStringList.AddStrings(Strings: TStrings);
var i:Integer;
begin
for I := 0 to Strings.Count - 1 do
      AddObject(Strings[I],i, Strings.Objects[I]);
end;

procedure TXMLHashStringList.BeginUpdate;
begin
if FUpdateCount = 0 then SetUpdateState(True);
  Inc(FUpdateCount);
end;

procedure TXMLHashStringList.Changed;
begin
 if (FUpdateCount = 0) and Assigned(FOnChange) then FOnChange(Self);
end;

procedure TXMLHashStringList.Changing;
begin
if (FUpdateCount = 0) and Assigned(FOnChanging) then FOnChanging(Self);
end;

procedure TXMLHashStringList.Clear;
var I:Longint;
begin
  if FCount <> 0 then
  begin
  Changing;
  For I:=0 to FHashTableSize-1 do
    FHashTable[i].List.Clear;
  FCount := 0;
  Changed;
  end;
end;

constructor TXMLHashStringList.Create(const AllowedClassType: TClass; aHashTableSize:Longint; aHashFunction:THashFunction);
var i:Longint;
begin
  inherited Create(AllowedClassType);
  FHashTableSize:=aHashTableSize;
  GetMem(FHashTable,FHashTableSize*SizeOf(THashItem));
  FillChar(PChar(FHashTable)^,FHashTableSize*SizeOf(THashItem),#0);
  {create one doubleLinked list for each enty in the Array}
  for i:=0 to FHashTableSize-1 do
     begin
     FHashTable[i].List:=TXMLDoubleLinkList.Create(AllowedClassType);
     FHashTable[i].List.CompareFunction:=CompareByName;
     end;
  {assign the hash function}
  FHashFunction:=aHashFunction;
  Duplicates:=dupAccept;


end;

function TXMLHashStringList.Delete(const S: String): TObject;
var strKey:string;
    i:Longint;
    List:TXMLDoubleLinkList;
    found:Boolean;
begin
Result:=nil;
{extract the key from the S string in case its in the form 'Name=Value'); }
strKey:=GetNamePart(S);
i:=FHashFunction(strKey,FHashTableSize);
List:=FHashTable[i].List;
found:=List.Search(0,S,nil);
if Found then
   Begin
   Result:=List.Delete;
   Dec(FCount);
   end;
end;

destructor TXMLHashStringList.Destroy;
var i:longint;
begin
inherited Destroy; //calls clear
for i:=0 to FHashTableSize-1 do
     begin
     FHashTable[i].List.Free;
     end;
FreeMem(FHashTable);
FOnChange := nil;
FOnChanging := nil;
end;



procedure TXMLHashStringList.EndUpdate;
begin
Dec(FUpdateCount);
  if FUpdateCount = 0 then SetUpdateState(False);
end;



function TXMLHashStringList.ForEachCall(
  const FunctionIterator: TFunctionIterator;
  const ForwardDirection: Boolean; const Data: Pointer): PDZItem;
var i:Longint;
begin
  i:=0;
  Result:=nil;
  While (Result=nil) and (i<FHashTableSize) do
    begin
     Result:=fHashTable[i].List.ForEachCall(FunctionIterator,ForwardDirection,Data);
     inc(i);
    end;
end;



function TXMLHashStringList.ForEachCall(
  const FunctionMethodIterator: TFunctionMethodIterator;
  const ForwardDirection: Boolean; const Data: Pointer): PDZItem;
var i:Longint;
begin
  i:=0;
  Result:=nil;
  While (Result=nil) and (i<FHashTableSize) do
    begin
     Result:=fHashTable[i].List.ForEachCall(FunctionMethodIterator,ForwardDirection,Data);
     inc(i);
    end;
end;

function TXMLHashStringList.GetObject(S: String): TObject;
var  item:PDZItem;
begin
  item:=find(S,false);
  if item<>nil then
    Result:=item^.FObject
  else Result:=nil;
end;

function TXMLHashStringList.GetTextString: string;
begin
end;

function TXMLHashStringList.GetValue(const Name: string): string;
var  item:PDZItem;
begin
  item:=find(Name,false);
  if item<>nil then
    Result:=GetValuePart(item^.FString)
  else Result:='';
end;

procedure TXMLHashStringList.SetObject(S: String; const Obj: TObject);
var  item:PDZItem;
begin
  item:=find(S,true);
  if item<>nil then
    item^.FObject:=Obj;
end;

procedure TXMLHashStringList.SetTextString(const S: string);
begin
{TODO 2 -oDean : gestione per textstring}
end;

procedure TXMLHashStringList.SetUpdateState(Updating: Boolean);
begin
if Updating then Changing else Changed;
end;

procedure TXMLHashStringList.SetValue(const Name, Value: string);
var  item:PDZItem;
begin
  item:=find(Name,true);
  if item<>nil then
    SetValuePart(item^.FString,Value)
end;




function TXMLHashStringList.Find(const S:string; const RaiseException:boolean):PDZItem;
var strKey:string;
    i:Longint;
    List:TXMLDoubleLinkList;
    found:Boolean;
begin
Result:=nil;
{extract the key from the S string in case its in the form 'Name=Value'); }
strKey:=GetNamePart(S);
i:=FHashFunction(strKey,FHashTableSize);
List:=FHashTable[i].List;
found:=List.Search(0,S,nil);
if Found then
   Result:=List.GetCursor
else begin
     if RaiseException then raise Exception.Create('Item not found');
{TODO 1 -oDean -cImportant:change with a custom exception}
     end;
end;

{returns -1 if item not found}
function TXMLHashStringList.GetVariantValue(const Name: string): Variant;
var  item:PDZItem;
begin
  item:=find(Name,false);
  if item<>nil then
    Result:=item^.FVariantValue
  else Result:=-1;
end;


procedure TXMLHashStringList.SetVariantValue(const Name: string;
  Value: Variant);
var  item:PDZItem;
begin
  item:=find(Name,true);
  if item<>nil then
    item^.FVariantValue:=Value;
end;




function TXMLHashStringList.GetShowStatistics: String;
var i:Longint;
    sum,counter:Longint;
    MaxChainLength:Longint;
begin
counter:=0;
sum:=0;
MaxChainLength:=0;
For i:=0 to FHashTableSize-1 do
    begin
    if FHashTable[i].List.Count>0 then
      begin
      sum:=sum+FHashTable[i].List.count;
      If FHashTable[i].List.count>MaxChainLength then MaxChainLength:=FHashTable[i].list.count;
      inc(counter);
      end;
    end;

Result:='Hash Table Size: '+IntToStr(FHashTableSize)+#10#13+
        'Number of Items: '+IntToStr(Count)+#10#13+
        'Number of chains: '+intToStr(Counter)+#10#13+
        'Average chain length: '+FloatToStr(Sum/Counter)+ #10#13+
        'Max chain length: '+IntToStr(MaxChainLength);

end;


procedure TXMLDoubleLinkList.LoadFromXML(sTagName, sXML: String);
var MNode,Child:TMiddleXNode;
holdTag,holdClassName,strField:String;
variantField:Variant;
objField:TPersistent;
objClass:TPersistentClass;
j,k:longint;
FMiddleXQuickRTTI:TMiddleXQuickRTTI;
begin
  inherited;
FMiddleXQuickRTTI:=TMiddleXQuickRTTI.Create;
Try
MNode:=TMiddleXNode.Create;
  MNode.XML:=sXML;
  clear;
  for j:= 0 to MNode.ChildNodeCount -1 do
    if assigned(MNode.childnode(j)) then
     begin
        Child:=MNode.childnode(j);
        strField:=Child.childNode(0).body;
        variantField:=Child.childNode(1).body;
        holdTag:='OBJECT';
        k:=Child.IndexByName(holdTag);
        if k>-1 then
          begin
          holdclassname:=Child.childNode(k).properties.values['TYPE'];
          ObjClass:= GetClass(holdClassname);
          if ObjClass<>nil then
            begin
            ObjField:=ObjClass.Create;
            FMiddleXQuickRTTI.rttiobject:=ObjField ;
            FMiddleXQuickRTTI.tagname:=holdtag;
            FMiddleXQuickRTTI.XML:=Child.childnode(k).xml ;
            InsertBeforeCursor(variantField,StrField,ObjField);
            end
          else raise exception.create('Class '+ holdclassName +' not registered.');
          end
        else InsertBeforeCursor(variantField,StrField,nil);;
     end;
MNode.Free;
finally
FMiddleXQuickRTTI.Free;
end;
end;

function TXMLDoubleLinkList.SaveToXML(sTagName: String): String;
var outhold, holdObj:String;
 FIteratorCursor:PDZItem;
 FMiddleXQuickRTTI:TMiddleXQuickRTTI;
begin
FMiddleXQuickRTTI:=TMiddleXQuickRTTI.create;
Try
outhold:='';
uniquestring(outhold);
outhold:='<'+sTagname+' TYPE="'+'TXMLDoubleLinkedList'+'"'+'>'+#13#10;

FIteratorCursor:=FBOL^.FNext;
   while (FIteratorCursor<>FEOL)  do
      begin
      holdObj:='';
      FMiddleXQuickRTTI.rttiobject:=TPersistent(FIteratorCursor.FObject);
      FMiddleXQuickRTTI.TagName := 'OBJECT';
      HoldObj:=FMiddleXQuickRTTI.XML;
      outhold:=outhold+'<ITEM>'
      +'<TEXT>'+ FIteratorCursor.FString+'</TEXT>'+#10
      +'<VARIANTVALUE>'+ VarToStr(FIteratorCursor.FVariantValue)+'</VARIANTVALUE>'
      +#10+HoldObj +'</ITEM>'+#10;
      FIteratorCursor:=FIteratorCursor^.FNext;;
      end;
 outhold:=outhold+'</'+sTagname+'>'+#13+#10;
 result:= outhold;
finally
FMiddleXQuickRTTI.free;
end;
end;

procedure TXMLHashStringList.LoadFromXML(sTagName, sXML: String);

var Child:TMiddleXNode;
holdTag,holdClassName,strField:String;
VariantField:Variant;
objClass:TPersistentClass;
obj:TPersistent;
j,k:longint;
Cache,FMiddleXQuickRTTI:TMiddleXQuickRTTI;
begin
  inherited;
FMiddleXQuickRTTI:=TMiddleXQuickRTTI.Create;
Cache:=TMiddleXQuickRTTI.Create;
Try
  FMiddleXQuickRTTI.XML:=sXML;
  clear;
  for j:= 0 to FMiddleXQuickRtti.Node.ChildNodeCount -1 do
    if assigned(FMiddleXQuickRTTI.Node.childnode(j)) then
     begin
        Child:=FMiddleXQuickRTTI.Node.childnode(j);
        strField:=Child.childNode(0).body;
        variantField:=Child.childNode(1).body;
        holdTag:='OBJECT';
        k:=Child.IndexByName(holdTag);
        if k>-1 then
          begin
          holdclassname:=Child.childNode(k).properties.values['TYPE'];
          ObjClass:= GetClass(holdClassname);
          if ObjClass<>nil then
            begin
            obj:=ObjClass.Create;
            Cache.rttiobject:=Obj ;
            Cache.tagname:=holdtag;
            Cache.XML:=Child.childnode(k).xml ;
            AddObject(StrField,variantField,cache.rttiobject);
            end
          else raise exception.create('Class '+ holdclassName +' not registered.');
          end
        else Add(strField,variantField);
     end;

finally
FMiddleXQuickRTTI.Free;
cache.Free;
end;


end;



function TXMLHashStringList.SaveToXML(sTagName: String): String;
var outhold, holdObj:String;
 FMiddleXQuickRTTI:TMiddleXQuickRTTI;
 i:Longint;
 chain:TXMLDoubleLinkList;
begin
FMiddleXQuickRTTI:=TMiddleXQuickRTTI.create;
Try
outhold:='';
uniquestring(outhold);
outhold:='<'+sTagname+' TYPE="'+'TXMLHashStringList'+'"'+'>'+#13#10;
for i:=0 to FHashTableSize-1 do
  begin
  chain:=HashTable^[i].List;

  if Chain.Count>0 then
    begin
    Chain.MoveToBOL;
    Chain.MoveNext;
    while (not Chain.IsEOL)  do
      begin
      holdObj:='';
      FMiddleXQuickRTTI.rttiobject:=TPersistent(Chain.Content);
      FMiddleXQuickRTTI.TagName := 'OBJECT';
      HoldObj:=FMiddleXQuickRTTI.XML;
      outhold:=outhold+'<ITEM>'+#13#10
      +'<TEXT>'+ Chain.StringContent+'</TEXT>'+#10
      +'<VARIANTVALUE>'+ VarToStr(Chain.VariantValueContent)+'</VARIANTVALUE>'+#10+HoldObj+'</ITEM>'+#10;
      Chain.MoveNext;
      end;
    end;
  end;
 outhold:=outhold+'</'+sTagname+'>'+#13+#10;
 result:= outhold;
finally
FMiddleXQuickRTTI.free;
end;
end;

initialization
 pItemsRepository:=nil;
 pListOfPages:=nil;
 numPages:=0;
 numItemsInRepository:=0;
finalization
 finalizeDZContainers;

end.
